# README

## Description
The project is the backend for the Openplanet Voting plugin on Trackmania. This stand is still in the early stages. With the plugin a player can give a rating to the currently driven map, see the average rating score and see how many players have voted for the track in total.

## Technical description
The client can connect to the socket server via `willers.digital` on port `3201`. After the first JSON request from the client the passed mapId is mapped on the client. All clients that are also mapped to the map will get all vote changes. The socket server communicates with the express REST server, which in turn communicates with the MySQL database. The socket server can tell from the request which REST service to invoke, so the client does not communicate directly with the REST service. Assuming the JSON request was submitted correctly.

If you just want to call the REST-Server you can do it with: `https://tm-voting.willers.digital`.

## REST Services
⚠ **All parameters MUST be done in the url query `/getVotes?map=blah`**

**GET /getVotes**<br>
Returns the current voting status for the current map.<br>

| Parameter 	| Description                         	| Example                     
|------------	|-------------------------------------	|-----------------------------
| map        	| The map UID to show the votes on it 	| `OkNwgzSwDLZSWSRPsUKY7EA1Cg5`

Example response:
```json
{
    "mapUid": "OkNwgzSwDLZSWSRPsUKY7EA1Cg5",
    "votes": 53,
    "average": 85,
    "lastVoteDate": "2021-08-23T00:28:16.273Z"
}
```

**GET /getPlayerVote**<br>
Returns the current voting status of a player for the current map.<br>

| Parameter 	| Description                         	| Example                     
|------------	|-------------------------------------	|-----------------------------
| map        	| The map UID to show the votes on it 	| `OkNwgzSwDLZSWSRPsUKY7EA1Cg5`
| player       	| The player login that he voted on it 	| `Jtmn3kBnSSadky_mLNhp_A`

Example response:
```json
{
    "mapUid": "OkNwgzSwDLZSWSRPsUKY7EA1Cg5",
    "playerLogin": "Jtmn3kBnSSadky_mLNhp_A",
    "vote": 50,
    "date": "2021-08-23 04:55:49.105"
}
```

**GET /mostVotedMaps**<br>
Returns the top most voted maps. in decreasing order (from the most voted map to the least voted map)<br>

There's no parameters for this request!

Example response:
```json
{
    "OkNwgzSwDLZSWSRPsUKY7EA1Cg5": 65,
    "7fsfRSUCQ7YwfBEdRk_GivW6qzj": 56,
    "KOylxZkny8RdOEFhchN1kG6Uoo1": 32,
    "n8KeykdPYEsJM6RPEZcQHVRVRIb": 32,
    "RfGQDoOnJ0FiswOBjpL3V7nkF0a": 32,
    "Iwo4gO_0dQ3FVQ1xeYjho5ZmLrf": 1,
}
```

**GET /mostPlayerVoted**<br>
Returns the top of the players who have voted the most maps. in decreasing order (from the most player who voted to the least)<br>

There's no parameters for this request!

Example response:
```json
{
    "Jtmn3kBnSSadky_mLNhp_A": 10,
    "W01C9MLeQH2zZ8v_P-gXvA": 8,
    "TIA7WqNETVyjWNj3RV1shQ": 5,
    "c5jutptORLinoaIUmVWscA": 2,
    "xTn1Lty3Tki8v7w3FBqVIA": 1
}
```

**POST /setVote**<br>
Writes the vote of a player to the respective map into the database table.<br />
⚠ You need an Authorization header with a valid token in order to do requests for this. After getting your token, set your Authorization value to `Basic yourtokenhere`

| Parameter 	| Description                         	| Example                     
|------------	|-------------------------------------	|-----------------------------
| map        	| The map UID to vote 	| `OkNwgzSwDLZSWSRPsUKY7EA1Cg5`
| player       	| The player login that it will vote 	| `Jtmn3kBnSSadky_mLNhp_A`
| vote       	| A number between 0 and 100 to define the vote 	| `56`

Example Response:
```json
{
    "mapUid": "OkNwgzSwDLZSWSRPsUKY7EA1Cg5",
    "player": "Jtmn3kBnSSadky_mLNhp_A",
    "vote": 65,
    "date": "2021-08-23T03:32:18.301Z",
    "type": "update" // "new" if it's a new vote, "update" if the player already voted on this map before
}
```

## Vote definition
As you can already see, you need to supply a number for the vote. The backend defines the votes as follows:

| Vote | Plugin 
| -----|:--:|
| 100 | +++ |
| 80 | ++ |
| 60 | + |
| 40 | - |
| 20 | -- |
| 0 | --- |

## Star definition
The following star scale was defined for the voting plugin:

| Scale | Stars 
| -----|:--:|
| 92-100 | 5 |
| 83-91 | 4.5 |
| 74-82 | 4 |
| 64-73 | 3.5 |
| 55-63 | 3 |
| 45-54 | 2.5 |
| 36-44 | 2 |
| 27-35 | 1.5 |
| 18-26 | 1 |
| 9-17 | 0.5 |
| 0-8 | 0 |

## Attention

My backend is 100% not secure for SQL-Injection and some other secure stuff. If some more people want to use it, I should protect it more.. but that take some time. You are welcome to help me with this. That would be a nice thing if we can introduce a global map vote functionality in a secure and highly available way. :)