const path = require('path'),
    scriptName = path.basename(__filename).replace(/\.js$/i,'');

/**
 * @param {import('express').Express} app
 * @param {import('mysql').Connection} sql
 */
module.exports = function(app, sql, token, errorHandler) {
    app.get('/'+scriptName, function(req, res) {
        const mapId = req.query.map;
        if (!mapId) {
            return errorHandler(res, 400, 'Missing map id');
        }
        sql.query('SELECT map, date, COUNT(*) AS count, round(avg(vote)) as average FROM votes WHERE map = ?', mapId, (err, sqlRes)=>{
            if (err) {
                console.log(err);
                errorHandler(res, 500, 'Internal server error');
            } else {
                if (sqlRes[0].map == null) {
                    return res.json({
                        mapUid: mapId,
                        votes: 0,
                        average: 0,
                        lastVoteDate: ""
                    });
                }
                // Get the latest date from an array
                const latestDate = new Date(Math.max.apply(null, sqlRes.map(function(e) {
                    return new Date(e.date);
                })));

                res.json({
                    mapUid: sqlRes[0].map,
                    votes: sqlRes[0].count,
                    average: sqlRes[0].average,
                    lastVoteDate: latestDate
                });
            }
        });
    });
};