const path = require('path'),
    scriptName = path.basename(__filename).replace(/\.js$/i,''),
    tmio = require('trackmania.io'),
    tmioPlayer = new tmio.Players();

/**
 * @param {import('express').Express} app
 * @param {import('mysql').Connection} sql
 */
module.exports = function(app, sql, tokenCheck, errorHandler) {
    app.post('/'+scriptName, function(req, res) {
        //tokenCheck(req, res, sql, ()=>{
            const mapId = req.query.map,
                player = req.query.player,
                vote = Number(req.query.vote);
            if (!mapId) return errorHandler(res, 400, 'Missing map id');
            if (!player) return errorHandler(res, 400, 'Missing player login');
            if (!vote) return errorHandler(res, 400, 'Missing vote');
            if (isNaN(vote)) return errorHandler(res, 400, 'Invalid vote');
            if (vote < 0 || vote > 100) return errorHandler(res, 400, 'Vote must be between 0 and 100');

            // check if player exists (tmio API check)
            tmioPlayer.player(player).then((playerData)=>{
                if (playerData.error) return errorHandler(res, 400, playerData.error);
            });

            // check if map exists (tmio API check)
            tmio.map(mapId).then((mapData)=>{
                if (mapData.error) return errorHandler(res, 400, mapData.error);
            });

            sql.query('SELECT * FROM votes WHERE map = ? AND player = ?', [mapId,player], (err, sqlRes)=>{
                if (err) {
                    console.log(err);
                    errorHandler(res, 500, 'Internal server error');
                } else {
                    const voteDate = new Date();
                    if (sqlRes.length === 0) { // Create a new vote for this player
                        sql.query('INSERT INTO votes (map, player, vote, date) VALUES (?, ?, ?, ?)', [mapId, player, vote, voteDate], (err)=>{
                            if (err) {
                                console.error(err);
                                errorHandler(res, 500, 'Internal server error');
                            } else {
                                res.json({
                                    mapUid: mapId,
                                    player: player,
                                    vote: vote,
                                    date: voteDate,
                                    type: "new"
                                });
                            }
                        });
                    } else { // Update the vote for this player
                        sql.query('UPDATE votes SET vote = ?, date = ? WHERE map = ? AND player = ?', [vote, voteDate, mapId, player], (err)=>{
                            if (err) {
                                console.error(err);
                                errorHandler(res, 500, 'Internal server error');
                            } else {
                                res.json({
                                    mapUid: mapId,
                                    player: player,
                                    vote: vote,
                                    date: voteDate,
                                    type: "update"
                                });
                            }
                        });
                    }
                }
            });
        //});
    });
};